﻿using System;

namespace Petzold.MusicSynthesis
{
    // Uses only integers in per-sample logic!
    public class AttackDecayEnvelope : IMonoSampleProvider
    {
        const long BITS16 = 1 << 16;
        const long BITS24 = 1 << 24;
        const long BITS32 = 1L << 32;

        int sampleRate;
        long volume = BITS16;  // ranges from 0x10000 to 0x100000000
        int direction;         // 1 for attack, -1 for decay, 0 for hold
        long upMultiplier;
        long downMultiplier;

        public AttackDecayEnvelope(int sampleRate)
        {
            this.sampleRate = sampleRate;
            this.AttackTime = TimeSpan.FromMilliseconds(100);
            this.DecayTime = TimeSpan.FromSeconds(1);
        }

        public IMonoSampleProvider Input { set; get; }

        public TimeSpan AttackTime { set; get; }

        public TimeSpan DecayTime { set; get; }

        public double Volume
        {
            get { return volume / (double)BITS32; }
        }

        public void Attack()
        {
            upMultiplier = (long)(BITS24 * Math.Pow(2, 16 / (sampleRate * this.AttackTime.TotalSeconds)));
            downMultiplier = (long)(BITS24 / Math.Pow(2, 16 / (sampleRate * this.DecayTime.TotalSeconds)));
            direction = 1;
        }

        public short GetNextSample()
        {
            short sample = 0;

            if (this.Input != null)
                sample = this.Input.GetNextSample();

            switch (direction)
            {
                case 1:     // attack
                    volume *= upMultiplier;
                    volume >>= 24;

                    if (volume >= BITS32)
                    {
                        volume = BITS32;
                        direction = -1;
                    }
                    break;

                case -1:    // decay
                    volume *= downMultiplier;
                    volume >>= 24;

                    if (volume <= BITS16)
                    {
                        volume = BITS16;
                        direction = 0;
                    }
                    break;

                case 0:
                    return 0;
            }
            return (short)((volume * sample) >> 32);
        }
    }
}
