﻿using System;

namespace Petzold.MusicSynthesis
{
    // Uses only integers in per-sample logic!
    public class IntegerMixerInput : IMixerInput
    {
        int iVolume;
        int iSpace;

        public IntegerMixerInput()
        {
            this.Volume = 1;
        }

        public IntegerMixerInput(IMonoSampleProvider input)
            : this()
        {
            this.Input = input;
        }

        public IMonoSampleProvider Input { set; get; }

        // Ranges from 0 to 1
        public double Volume 
        { 
            set { iVolume = (int)(65536.0 * Math.Min(1, Math.Max(0, value))); }
            get { return iVolume / 65536.0; }
        }

        // Ranges from -1 (left) to 0 (center) to 1 (right)
        public double Space 
        { 
            set { iSpace = (int)(65536.0 * Math.Min(1, Math.Max(-1, value))); }
            get { return iSpace / 65536.0; } 
        }

        public AudioSample GetNextSample()
        {
            short sample = 0;

            if (iVolume == 0)
                return new AudioSample();

            if (this.Input != null)
                sample = Input.GetNextSample();

            if (sample == 0)
                return new AudioSample();

            return new AudioSample
            {
                Left = (short)(((long)sample * iVolume * (65536 - iSpace)) >> 33),
                Right = (short)(((long)sample * iVolume * (65536 + iSpace)) >> 33)
            };
        }
    }
}
