﻿using System.Collections.Generic;

namespace Petzold.MusicSynthesis
{
    public class Mixer : IStereoSampleProvider
    {
        public Mixer()
        {
            this.Inputs = new List<IMixerInput>();
            this.Volume = 1;
        }

        public IList<IMixerInput> Inputs { protected set; get; }

        public double Volume { set; get; }

        public AudioSample GetNextSample()
        {
            int left = 0;
            int right = 0;

            foreach (IMixerInput mixerInput in this.Inputs)
            {
                AudioSample audioSample = mixerInput.GetNextSample();
                left += audioSample.Left;
                right += audioSample.Right;
            }

            // Quicken the calculation
            if (this.Volume == 1)
            {
                return new AudioSample
                {
                    Left = (short)(left / this.Inputs.Count),
                    Right = (short)(right / this.Inputs.Count)
                };
            }

            return new AudioSample
            {
                Left = (short)(this.Volume * left / this.Inputs.Count),
                Right = (short)(this.Volume * right / this.Inputs.Count)
            };
        }
    }
}
