﻿using System;

namespace Petzold.MusicSynthesis
{
    public class MixerInput : IMixerInput
    {
        double volume;
        double space;

        public MixerInput()
        {
            this.Volume = 1;
        }

        public MixerInput(IMonoSampleProvider input)
            : this()
        {
            this.Input = input;
        }

        public IMonoSampleProvider Input { set; get; }

        // Ranges from 0 to 1
        public double Volume 
        { 
            set { volume = Math.Min(1, Math.Max(0, value)); }
            get { return volume; }
        }

        // Ranges from -1 (left) to 0 (center) to 1 (right)
        public double Space 
        { 
            set { space = Math.Min(1, Math.Max(-1, value)); }
            get { return space; } 
        }

        public AudioSample GetNextSample()
        {
            short sample = 0;

            if (this.Input != null)
                sample = Input.GetNextSample();

            return new AudioSample
            {
                Left = (short)(sample * this.Volume * (1 - this.Space) / 2),
                Right = (short)(sample * this.Volume * (1 + this.Space) / 2)
            };
        }
    }
}
