﻿using System;

namespace Petzold.MusicSynthesis
{
    public abstract class Oscillator : IMonoSampleProvider
    {
        int sampleRate;
        double angleIncrement;
        double angle;

        private Oscillator()
        {
        }

        public Oscillator(int sampleRate)
            : this()
        {
            this.sampleRate = sampleRate;
        }

        public double Frequency 
        { 
            set
            {
                angleIncrement = 2 * Math.PI * value / sampleRate;
            }
            get
            {
                return angleIncrement * sampleRate / 2 / Math.PI;
            }
        }

        public short GetNextSample()
        {
            short sample = GetNextSample(angle);
            angle = (angle + angleIncrement) % (2 * Math.PI);
            return sample;
        }

        // angle argument ranges from 0 to 2*PI
        public abstract short GetNextSample(double angle);
    }
}
