﻿using System;

namespace Petzold.MusicSynthesis
{
    public struct Pitch
    {
        static readonly double semitone = Math.Pow(2, 1.0 / 12);
        static readonly double middleC = 440 * Math.Pow(semitone, 3) / 2;

        // Middle C is 64
        public Pitch(int midiNumber) :
            this((Note)(midiNumber % 12), midiNumber / 12 - 1)
        {
        }

        public Pitch(Note note, int octave)
            : this()
        {
            this.Note = note;
            this.Octave = octave;
            this.MidiNumber = 12 * (this.Octave + 1) + (int)this.Note;
            this.Frequency = middleC * Math.Pow(semitone, (int)Note) * Math.Pow(2, Octave - 4);
        }

        public Note Note { private set; get; }
        public int Octave { private set; get; }
        public int MidiNumber { private set; get; }
        public double Frequency { private set; get; }

        public bool IsNatural
        {
            get
            {
                return Note == Note.C ||
                       Note == Note.D ||
                       Note == Note.E ||
                       Note == Note.F ||
                       Note == Note.G ||
                       Note == Note.A ||
                       Note == Note.B;
            }
        }

        public override string ToString()
        {
            string note = this.Note.ToString();

            return String.Format("{0}{1}{2} {3} ({4}) {5}Hz", note[0], note.EndsWith("sharp") ? "\x266F" : "", 
                                                                       note.EndsWith("flat") ? "\x266D" : "", 
                                                              this.Octave, this.MidiNumber, this.Frequency);
        }
    }
}
