﻿using System;
using Petzold.MusicSynthesis;

namespace Theremin
{
    public class ThereminOscillator : Oscillator
    {
        readonly double ampStep;
        readonly double freqStep;

        public const double MIN_AMPLITUDE = 0.0001;

        public ThereminOscillator(int sampleRate)
            : base(sampleRate)
        {
            ampStep = 1 + 0.12 * 1000 / sampleRate;     // ~1 db per msec
            freqStep = 1 + 0.005 * 1000 / sampleRate;   // ~10 cents per msec
        }

        public double Amplitude { set; get; }
        public double DestinationAmplitude { get; set; }
        public double DestinationFrequency { set; get; }

        public override short GetNextSample(double angle)
        {
            this.Frequency *= this.Frequency < this.DestinationFrequency ? freqStep : 1 / freqStep;
            this.Amplitude *= this.Amplitude < this.DestinationAmplitude ? ampStep : 1 / ampStep;
            this.Amplitude = Math.Max(MIN_AMPLITUDE, Math.Min(1, this.Amplitude));
            return (short)(short.MaxValue * this.Amplitude * Math.Sin(angle));
        }
    }
}
